/*
 * mh_connected_client.c
 *
 *  Created on: 05-Sep-2014
 *      Author: sach.sharma
 */
#include <time.h>

#include "mh_view_main.h"
#include "mobile_hotspot.h"
#include "mh_string.h"

static void __remove_conn_clients(mh_appdata_t *ad)
{
	__MOBILE_AP_FUNC_ENTER__;

	mh_main_view_t *mv = &ad->main;
	int connected_clients = 0;
	int i = 0;

	if (_get_no_of_connected_device(ad, &connected_clients,
					TETHERING_TYPE_ALL) == EINA_FALSE)
		ERR("Getting the number of connected device is failed");

	for (i = 0; i < MAX_CONNECTED_STA; i++)
		_free_genlist_item(&ad->main.station_items[i]);

	_free_genlist_itc(&mv->dev_itc[TETHERING_TYPE_WIFI]);
	_free_genlist_itc(&mv->dev_itc[TETHERING_TYPE_BT]);
	_free_genlist_itc(&mv->dev_itc[TETHERING_TYPE_USB]);

	_stop_update_device_conn_time(ad);

	__MOBILE_AP_FUNC_EXIT__;
}

static void __update_connected_clients(mh_appdata_t *ad)
{
	__MOBILE_AP_FUNC_ENTER__;

	if (ad == NULL) {
		ERR("Invalid param\n");
		return;
	}

	Elm_Object_Item *item = NULL;
	tethering_type_e connection_type = 0;
	GSList *l = NULL;
	tethering_client_h *handle;
	int count = 0;
	mh_main_view_t *main_view = &(ad->main);
	int i = 0;
	int connected_clients = 0;

	if (_get_no_of_connected_device(ad, &connected_clients,
					TETHERING_TYPE_ALL) == EINA_FALSE)
		ERR("Getting the number of connected device is failed");

	for (i = 0; i < MAX_CONNECTED_STA; i++)
		_free_genlist_item(&ad->main.station_items[i]);

	for (l = ad->client_list; l != NULL; l = g_slist_next(l)) {
		handle = (tethering_client_h *)l->data;
		tethering_client_get_tethering_type(handle, &connection_type);

		item = elm_genlist_item_append(main_view->genlist,
				main_view->dev_itc[connection_type],
				(void *)handle,
				NULL, ELM_GENLIST_ITEM_NONE,
				NULL, NULL);
		if (item == NULL) {
			ERR("elm_genlist_item_append is failed\n");
			continue;
		}
		main_view->station_items[count++] = item;
		elm_genlist_item_select_mode_set(item, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
	}
	main_view->no_of_clients = count;
	_start_update_device_conn_time(ad);
	__MOBILE_AP_FUNC_EXIT__;
	return;
}

void _update_conn_clients(mh_appdata_t *ad)
{
	__MOBILE_AP_FUNC_ENTER__;

	if (ad == NULL) {
		ERR("Invalid param\n");
		return;
	}
	int connected_clients = 0;
	if (_get_no_of_connected_device(ad, &connected_clients,
					TETHERING_TYPE_ALL) == EINA_FALSE)
		ERR("Getting the number of connected device is failed");

	if (connected_clients == 0)
		__remove_conn_clients(ad);
	else
		__update_connected_clients(ad);

	__MOBILE_AP_FUNC_EXIT__;
	return;
}

static char *__get_diff_time(time_t connection_time)
{
	time_t current_time;
	char buf[MH_LABEL_LENGTH_MAX] = {0, };
	int day;
	int hour;
	int min;
	int sec;
	double diff;

	time(&current_time);
	diff = difftime(current_time, connection_time);
	day = diff / (60 * 60 * 24);
	diff = diff - (day * 60 * 60 * 24);
	hour = diff / (60 * 60);
	diff = diff - (hour * 60 * 60);
	min = diff / 60;
	diff = diff - (min * 60);
	sec = diff;

	if (day > 0)
		hour = hour + day * 24;

	if (hour > 0)
		snprintf(buf, sizeof(buf), "%02d:%02d:%02d", hour, min, sec);
	else
		snprintf(buf, sizeof(buf), "%02d:%02d", min, sec);

	return g_strdup(buf);
}

static char *__gl_get_dev_label(void *data, Evas_Object *obj, const char *part)
{
	__MOBILE_AP_FUNC_ENTER__;

	tethering_client_h client = (tethering_client_h)data;
	char *name = NULL;
	char *dev_name = NULL;
	time_t tm;

	if (data == NULL || obj == NULL || part == NULL) {
		ERR("Invalid param\n");
		return NULL;
	}

	if (!strcmp("elm.text", part)) {
		tethering_client_get_name(client, &name);
		if (NULL == name)
			return NULL;

		if (!strcmp(name, "UNKNOWN")) {
			free(name);
			return strdup(STR_NO_NAME);
		}

		dev_name = elm_entry_utf8_to_markup(name);
		free(name);
		return dev_name;
	} else if (!strcmp("elm.text.sub", part)) {
		tethering_client_get_time(client, &tm);
		return __get_diff_time(tm);
	}

	__MOBILE_AP_FUNC_EXIT__;
	return NULL;
}

static Evas_Object *__gl_get_dev_wifi_icon(void *data, Evas_Object *obj,
							const char *part)
{
	Evas_Object *icon = NULL;

	if (!strcmp("elm.swallow.icon", part)) {
		icon = elm_icon_add(obj);
		elm_image_file_set(icon, EDJDIR"/"TETHERING_IMAGES_EDJ, WIFI_ICON);
		evas_object_color_set(icon, 2, 61, 132, 204);
		evas_object_size_hint_min_set(icon, ELM_SCALE_SIZE(DEFAULT_ICON_SIZE), ELM_SCALE_SIZE(DEFAULT_ICON_SIZE));
	}
	return icon;
}

static Evas_Object *__gl_get_dev_usb_icon(void *data, Evas_Object *obj,
							const char *part)
{
	Evas_Object *icon = NULL;

	if (!strcmp("elm.swallow.icon", part)) {
		icon = elm_icon_add(obj);
		elm_image_file_set(icon, EDJDIR"/"TETHERING_IMAGES_EDJ, USB_ICON);
		evas_object_color_set(icon, 2, 61, 132, 204);
		evas_object_size_hint_min_set(icon, ELM_SCALE_SIZE(DEFAULT_ICON_SIZE), ELM_SCALE_SIZE(DEFAULT_ICON_SIZE));
	}
	return icon;
}

static Evas_Object *__gl_get_dev_bt_icon(void *data, Evas_Object *obj,
							const char *part)
{
	Evas_Object *icon = NULL;

	if (!strcmp("elm.swallow.icon", part)) {
		icon = elm_image_add(obj);
		elm_image_file_set(icon, EDJDIR"/"TETHERING_IMAGES_EDJ, BT_ICON);
		evas_object_color_set(icon, 2, 61, 132, 204);
		evas_object_size_hint_min_set(icon, ELM_SCALE_SIZE(DEFAULT_ICON_SIZE), ELM_SCALE_SIZE(DEFAULT_ICON_SIZE));
	}
	return icon;
}

static void __set_genlist_itc(mh_appdata_t *ad)
{
	__MOBILE_AP_FUNC_ENTER__;

	ad->main.dev_itc[TETHERING_TYPE_WIFI] = elm_genlist_item_class_new();
	ad->main.dev_itc[TETHERING_TYPE_WIFI]->item_style = MH_GENLIST_2LINE_TOP_TEXT_ICON_STYLE;
	ad->main.dev_itc[TETHERING_TYPE_WIFI]->func.text_get = __gl_get_dev_label;
	ad->main.dev_itc[TETHERING_TYPE_WIFI]->func.content_get = __gl_get_dev_wifi_icon;
	ad->main.dev_itc[TETHERING_TYPE_WIFI]->func.state_get = NULL;
	ad->main.dev_itc[TETHERING_TYPE_WIFI]->func.del = NULL;

	ad->main.dev_itc[TETHERING_TYPE_USB] = elm_genlist_item_class_new();
	ad->main.dev_itc[TETHERING_TYPE_USB]->item_style = MH_GENLIST_2LINE_TOP_TEXT_ICON_STYLE;
	ad->main.dev_itc[TETHERING_TYPE_USB]->func.text_get = __gl_get_dev_label;
	ad->main.dev_itc[TETHERING_TYPE_USB]->func.content_get = __gl_get_dev_usb_icon;
	ad->main.dev_itc[TETHERING_TYPE_USB]->func.state_get = NULL;
	ad->main.dev_itc[TETHERING_TYPE_USB]->func.del = NULL;

	ad->main.dev_itc[TETHERING_TYPE_BT] = elm_genlist_item_class_new();
	ad->main.dev_itc[TETHERING_TYPE_BT]->item_style = MH_GENLIST_2LINE_TOP_TEXT_ICON_STYLE;
	ad->main.dev_itc[TETHERING_TYPE_BT]->func.text_get = __gl_get_dev_label;
	ad->main.dev_itc[TETHERING_TYPE_BT]->func.content_get = __gl_get_dev_bt_icon;
	ad->main.dev_itc[TETHERING_TYPE_BT]->func.state_get = NULL;
	ad->main.dev_itc[TETHERING_TYPE_BT]->func.del = NULL;

	__MOBILE_AP_FUNC_EXIT__;
}

static void __scroll_start_cb(void *data, Evas_Object *obj, void *event_info)
{
	__MOBILE_AP_FUNC_ENTER__;

	_stop_update_device_conn_time((mh_appdata_t *)data);
	__MOBILE_AP_FUNC_EXIT__;
}

static void __scroll_stop_cb(void *data, Evas_Object *obj, void *event_info)
{
	__MOBILE_AP_FUNC_ENTER__;

	_start_update_device_conn_time((mh_appdata_t *)data);
	__MOBILE_AP_FUNC_EXIT__;
}

static void __append_clients_genlist(mh_appdata_t *ad)
{
	__MOBILE_AP_FUNC_ENTER__;

	if (ad == NULL) {
		ERR("ad is NULL\n");
		return;
	}

	Elm_Object_Item *item = NULL;
	tethering_type_e connection_type = 0;
	GSList *l = NULL;
	tethering_client_h *handle;
	int count = 0;

	mh_main_view_t *mv = &ad->main;

	__set_genlist_itc(ad);

	for (l = ad->client_list; l != NULL; l = g_slist_next(l)) {
		handle = (tethering_client_h *)l->data;
		tethering_client_get_tethering_type(handle, &connection_type);

		item = elm_genlist_item_append(mv->genlist,
				mv->dev_itc[connection_type],
				(void *)handle,
				NULL, ELM_GENLIST_ITEM_NONE,
				NULL, NULL);
		if (item == NULL) {
			ERR("elm_genlist_item_append is failed\n");
			continue;
		}
		mv->station_items[count++] = item;
		elm_genlist_item_select_mode_set(item, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
	}
	evas_object_smart_callback_add(mv->genlist, "scroll,anim,start",
				__scroll_start_cb, ad);
	evas_object_smart_callback_add(mv->genlist, "scroll,drag,start",
				__scroll_start_cb, ad);
	evas_object_smart_callback_add(mv->genlist, "scroll,anim,stop",
				__scroll_stop_cb, ad);
	evas_object_smart_callback_add(mv->genlist, "scroll,drag,stop",
				__scroll_stop_cb, ad);

	mv->no_of_clients = count;
	_start_update_device_conn_time(ad);

	__MOBILE_AP_FUNC_EXIT__;
}

void _add_connected_clients(mh_appdata_t *ad)
{
	DBG("+\n");
	if (ad == NULL) {
		ERR("ad is NULL\n");
		return;
	}


	__append_clients_genlist(ad);

	DBG("-\n");
}
